@@ -1166,6 +1166,26 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         '''
         return getattr(self, '_rfm_fixt_variant', None)
 
+    def set_var_default(self, name, value):
+        '''Set the default value of a variable if variable is undefined.
+
+        A variable is undefined if it is declared and required and no value is
+        yet assigned to it.
+
+        :param name: The name of the variable.
+        :param value: The value to set the variable to.
+        :raises ValueError: If the variable does not exist
+
+        .. versionadded:: 3.10.1
+
+        '''
+        var_space = type(self).var_space
+        if name not in var_space:
+            raise ValueError(f'no such variable: {name!r}')
+
+        if not var_space[name].is_defined():
+            setattr(self, name, value)
+
     @property
     def perfvalues(self):
         return util.MappingView(self._perfvalues)
@@ -1498,8 +1518,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         self.logger.debug(f'Cloning URL {url} into stage directory')
         osext.git_clone(
             self.sourcesdir, self._stagedir,
-            # FIXME: cast to float explicitly due to GH #2246
-            timeout=float(rt.runtime().get_option('general/0/git_timeout'))
+            timeout=rt.runtime().get_option('general/0/git_timeout')
         )
 
     @final
