@@ -339,10 +339,6 @@ class AsynchronousExecutionPolicy(ExecutionPolicy, TaskEventListener):
                     'general/0/pipeline_timeout'
                 )
 
-                # FIXME: Always convert due to #GH 2246
-                if timeout is not None:
-                    timeout = float(timeout)
-
                 self._advance_all(self._current_tasks, timeout)
                 if self._pipeline_statistics:
                     num_retired = len(self._retired_tasks)
@@ -583,7 +579,8 @@ class AsynchronousExecutionPolicy(ExecutionPolicy, TaskEventListener):
         pass
 
     def on_task_skip(self, task):
-        pass
+        msg = str(task.exc_info[1])
+        self.printer.status('SKIP', msg, just='right')
 
     def on_task_failure(self, task):
         self._num_failed_tasks += 1
